# Digital Formulator and Self-Driving Tableting DataFactory

This repository contains the code and data for the paper "Accelerated Medicines Development using a Digital Formulator and a Self-Driving Tableting DataFactory" published in Nature Communications.

![System Overview](image.png)

## Repository Structure
- `data/`: Contains raw and processed test datasets
- `pipelines/`: Core workflow scripts for end-to-end processes (model prediction and insilico digital formulator)
- `src/`: Source code modules for hybrid system of models
- `saved_model/`: Directory for training data, pre-trained models, and results

## System Requirements

### Software Dependencies
- Python 3.8+ (tested on 3.8, 3.9, 3.10)
- keras==2.10.0
- matplotlib==3.8.0
- numpy==1.24.3
- pandas==2.2.2
- scikit_learn==1.3.0
- tensorflow==2.10.0
- typing_extensions==4.12.2
- xgboost==2.1.0
- seaborn
- pathlib
- psynlig

### Operating Systems
- Compatible with Windows 10/11, macOS (12.0+), and Linux (Ubuntu 20.04+)
- All code tested on Windows 11

### Hardware Requirements
- RAM: Minimum 8GB, recommended 16GB+
- Storage: At least 5GB free space
- No specialised hardware (e.g., GPUs) required for base functionality

## Installation Guide

### Installation Steps
1. Clone the repository:
```bash
git clone https://github.com/username/cmac-self-driving-tabletting-datafactory.git
cd cmac-self-driving-tabletting-datafactory
```

2. Create and activate a virtual environment (optional but recommended):
```bash
python -m venv venv
# On Windows
venv\Scripts\activate
# On macOS/Linux
source venv/bin/activate
```

3. Install required packages:
```bash
pip install -r requirements.txt
```

### Typical Installation Time
- Complete setup takes approximately 5-10 minutes on a standard desktop computer with a broadband internet connection.

## Data Cleaning and Preprocessing

The data preparation pipeline consists of the following steps:

1. **Data Ingestion** (`src/data_ingestion.py`): 
   - Loads raw data from Excel files in the `/saved_model` directory
   - Performs initial validation checks on data format and completeness

2. **Data Cleaning** (`src/data_cleaning.py`): 
   - Removes outliers using z-score analysis as well as missing values

3. **Feature Engineering** (`src/model_dev.py`):
   - Extracts input parameters for mixture and process models 
   - Calculates principal components from particle size and aspect ratio distributions
   - Computes blend properties (e.g., true density, bulk density, flowability) followed by tablet properties (porosity, tensile strength)

4. **Data Saving** (`src/data_saving.py`):
   - Exports processed datasets in standardised formats (.pkl files)
   - Creates data versioning (YYYMMDD_HR_data) for reproducibility

The complete pipeline can be executed through the scripts in the `pipelines` directory, with `loading_data.py` serving as the entry point for loading data, `hybrid_machine.py` for executing hybrid system of models and making predictions, and `insilico_formulation_optimisation.py` serving as the digital formulator to optimise formulation settings.

## Demo

### Running the Demo
To run a demonstration of the Digital Formulator:

```bash
python pipelines/insilico_formulation_optimisation.py 
```

This will execute the formulation optimization for paracetamol (SP) with a 20% drug loading.

### Expected Output
The demo will produce:
- Console output showing optimisation progress and final results
- A visualization of the optimal formulation composition
- Prediction of tablet properties (porosity, tensile strength, FFC)

### Expected Runtime
- The complete demo takes approximately 30 minutes on a standard desktop computer (Intel i5/i7 or equivalent, 16GB RAM).

## Instructions for Use

### Digital Formulator

To optimise a tablet formulation for a specific API and drug loading:

```bash
python pipelines/insilico_formulation_optimisation.py 
```

### Hybrid Machine Learning System

To run the hybrid system of models:

```bash
python pipelines/hybrid_machine.py 
```

## Additional Information

### License
This project is licensed under the Creative Commons Attribution Non Commercial 4.0 International License - see the LICENSE file for details.

### Contact
For questions or support, please contact the corresponding author: daniel.markl@strath.ac.uk