# -*- coding: utf-8 -*-
"""
Created on Mon Dec  9 11:13:16 2024

@author: moham
"""
import sys
from pathlib import Path
import numpy as np
import matplotlib.pyplot as plt
from steps.model_run import single_run_synthetised
from pymoo.core.problem import ElementwiseProblem
from pymoo.core.variable import Real, Choice
from pymoo.visualization.scatter import Scatter
from pymoo.algorithms.moo.nsga2 import NSGA2
from pymoo.core.mixed import MixedVariableMating, MixedVariableSampling, MixedVariableDuplicateElimination
from pymoo.optimize import minimize
from pymoo.core.callback import Callback
from pymoo.algorithms.soo.nonconvex.ga import GA

sys.path[0] = str(Path(sys.path[0]).parent)

class MixedVarsProblem(ElementwiseProblem):

    def __init__(self, **kwargs):

        variables = dict()

        # variables["x01"] = Real(bounds=(0.08, 0.3))
        variables["x01"] = Real(bounds=(0.0, 1-0.06-0.1))
        # variables[f"x03"] = Real(bounds=(0.0, 1.0))
        variables["x02"] = Choice(options=['ma1', 'la1', 'la3', 'mc5', 'mc6', 'mc7'])
        variables["x03"] = Choice(options=['ma1', 'la1', 'la3', 'mc5', 'mc6', 'mc7'])
        # ['ma1', 'ma4', 'ma6', 'la1', 'la3', 'mc5', 'mc6', 'mc7']
        variables["x04"] = Real(bounds=(90, 400))

        super().__init__(vars=variables, n_obj=1, n_constr=2, **kwargs)

    def _evaluate(self, x, out, *args, **kwargs):
        # x = np.array([x[f"x{k:02}"] for k in range(1, 7)])

        input2, input3, input4, input5 = x["x01"], x["x02"], x["x03"], x["x04"]
        input1=0.2
        # print(input1, input2, input3, input4, input5, input6)
        
        fractions=[input1, 0.04, input2, 1-input1-input2-0.06, 0.02]
        components=["sp1", "cc1", str(input3), str(input4), "ms1"]
        titles=["API", "disintegrant", "filler 1", "filler 2","lubricant"]
        f, p, porosity, tensile_strength = single_run_synthetised(titles,
                                                                  components,
                                                                  fractions, 
                                                                  cp=float(input5))
        
        objective1 = -f.ffc

        constraint1 = 2.05 - (p.tensile_mean - 0.2*p.tensile_std)
        constraint2 = 0.155 - (p.porosity_mean)
        
        out["G"] = [constraint1, constraint2, ]
        out["F"] = [objective1]

    
problem = MixedVarsProblem()

for name, var in problem.vars.items():
    print(name, var)

class MyCallback(Callback):

    def __init__(self) -> None:
        super().__init__()
        self.data["best"] = []
        self.data["iter"] = 0
        self.figure, self.ax = plt.subplots(figsize=(10, 8))
        
    def notify(self, algorithm):
        self.data["best"].append(algorithm.pop.get("F").min())
        self.data["iter"] += 1
         
        # to run GUI event loop
        plt.ion()
         
        # here we are creating sub plots
        ax1 = self.ax.scatter(np.arange(self.data["iter"])+1, self.data["best"], s=75, c="blue")
         
        # setting title
        plt.title("Optimisation Performance", fontsize=20)
         
        # setting x-axis label and y-axis label
        plt.xlabel("Iteration Number", fontsize=20)
        plt.ylabel("Objective Value", fontsize=20)
        plt.xticks(fontsize=20)
        plt.yticks(fontsize=20)
     
        self.figure.canvas.draw()
        self.figure.canvas.flush_events()            



################### NSGA2
algorithm = NSGA2(pop_size=20,
                  sampling=MixedVariableSampling(),
                  mating=MixedVariableMating(eliminate_duplicates=MixedVariableDuplicateElimination()),
                  eliminate_duplicates=MixedVariableDuplicateElimination(),
                  # callback=MyCallback(),
                  )

res = minimize(problem,
                algorithm,
                # termination=('n_gen', 100), # for multi objective NSGA2
                # termination=termination, # for robust termination
                termination=('n_evals', 200), # for single objective optuna
                seed=123,
                verbose=True,
                callback=MyCallback(),
                return_least_infeasible=True,
                )
print("Best solution found: \nX = %s\nF = %s\nCV = %s" % (res.X, res.F, res.CV))


def evalopt(x):
    try:
        x = x[0]
    except:
        x = x   
    input2, input3, input4, input5 = x["x01"], x["x02"], x["x03"], x["x04"]
    input1=0.2
    fractions=[input1, 0.04, input2, 1-input1-input2-0.06, 0.02]
    print(fractions)
    titles=["API", "disintegrant", "filler 1", "filler 2","lubricant"]
    print(titles)
    components=["sp1", "cc1", str(input3), str(input4), "ms1"]
    print(components)
    f, p, porosity, tensile_strength = single_run_synthetised(titles,
                                                              components,
                                                              fractions, 
                                                              cp=float(input5))
    print(input5)
    print(f.ffc)
    print(p.tensile_mean)
    print(p.tensile_std)
    print(p.porosity_mean)
    print(p.porosity_std)
    print(porosity)
    print(tensile_strength)
    return f, p

optim_resX = res.X #[-1]
f, p = evalopt(optim_resX)
