import os

class Config():
    """Model Configurations"""
    def __init__(self) -> None:
        try:
            username = os.environ["USERNAME"]  
        except: 
            username = "moham"   
            
        self.saved_models_path: str = "./saved_model/20240724_16_data/"
        
        self.raw_data_path: str = 'C:/Users/' + username + '/University of Strathclyde/SIPBS_CMAC_Product_Data - General/Data management/01_raw_materials/raw_material_master_file.xlsx'
        self.raw_data_sheet_name: str = 'Materials database'
        self.raw_data_skip_rows: int = 3
        
        self.blend_data_path: str = 'C:/Users/' + username + '/University of Strathclyde/SIPBS_CMAC_Product_Data - General/Data management/02_blend_data/blend_experiment_planning_master_file.xlsx'
        self.blend_data_sheet_name: str = 'Blend formulation experiments'
        self.blend_data_skip_rows: int = 3
        
        self.tablet_data_path: str = 'C:/Users/' + username + '/University of Strathclyde/SIPBS_CMAC_Product_Data - General/Data management/03_tablets/TabletData_master.xlsx'
        self.tablet_data_sheet_name: str = 'Sheet1'
        self.tablet_data_skip_rows: int = 1
        
        self.raw_PSD_num_path = "C:/Users/" + username + "/University of Strathclyde/SIPBS_CMAC_Product_Data - General/Data management/01_raw_materials/particle_size/raw_data/PSD number based distributions/"
        self.raw_AR_path = "C:/Users/" + username + "/University of Strathclyde/SIPBS_CMAC_Product_Data - General/Data management/01_raw_materials/particle_shape/raw_data/particle aspect ratio distributions/"
        self.blend_PSD_num_path = "C:/Users/" + username + "/University of Strathclyde/SIPBS_CMAC_Product_Data - General/Data management/02_blend_data/particle_size/raw_data/blend PSD number based distributions/"
        self.blend_AR_path = "C:/Users/" + username + "/University of Strathclyde/SIPBS_CMAC_Product_Data - General/Data management/02_blend_data/particle_shape/raw_data/particle aspect ratio distributions/"
        
        self.alternative_raw_dict = {"la6": "la1",
                                     "ms2": "ms1",
                                     "ms5": "ms1",
                                     "mc2": "mc1",
                                     "ma2": "ma1",
                                     "mc6": "mc1",
                                     "la4": "la1",
                                     "sp3": "sp1",
                                     "sp4": "sp1",
                                     "sp5": "sp1",
                                     "gr1": "gr2",
                                     "sp2": "sp1",
                                     "ms4": "ms1",
                                     "la5": "la1",
                                     "mc4": "mc1",
                                     # "ib1": "ib2",
                                     }
        
        self.csd_features = ["packing_coeff", 
                              "void_volume", 
                              "ave hb donors", 
                              "ave hb acceptors",
                              "ave rugosity", 
                              "ave surface charge",
                              "S/M", 
                              "M/L", 
                              "hbound_dim"]
        
        self.blend_comp_list = ["lubricant", "API", "disintegrant", "filler 1", "filler 2"]
        self.tablet_comp_list = ["API", "disintegrant", "filler 1", "filler 2", "excipient 3", "excipient 4", "lubricant"]
        
        self.tapped_density_features = ['PSD PC1', 'PSD PC2', 'PSD PC3', 'AR PC1', 'AR PC2', 'AR PC3', 'True Density Model', "BD Model", "TD"]
        self.tapped_density_response = 'TD'
        
        self.flowability_table_features = ["Consilidation pressure", "MPS", "UYS", "Gradient", "AIF", "C", "RHOB", "Eff. AoIF", "FFC"]
        self.ffc_model_features = ["API", 'PSD PC1', 'PSD PC2', 'PSD PC3', "AR PC1", "AR PC2", "AR PC3", 'True Density Model', "BD Model", "Consilidation pressure", "FFC"]
        self.ffc_response = "FFC"
        self.eaoif_model_features = ["API",'PSD PC1', 'PSD PC2', 'PSD PC3', "AR PC1", "AR PC2", "AR PC3", 'True Density Model', "BD Model", "Consilidation pressure", "Eff. AoIF"]
        self.eaoif_response = "Eff. AoIF"
        
        
        self.blends_base_features = ["PSD PC1", "PSD PC2", "PSD PC3", "AR PC1", "AR PC2", "AR PC3", "True Density Model", "BD Model"]
        self.process_base_features = ["API conc"] + self.blends_base_features + ['Compression Pressure (Mpa)']

        self.n_ensembles = 1
        
        self._name_df_raw = "df_raw"
        self._name_df_blend = "df_blend" 
        self._name_df_tablet = "df_tablet"
        
        
        self._name_ar_raw = "ar_raw"
        self._name_psd_raw = "psd_raw"
        self._name_ar_blend = "ar_blend"
        self._name_psd_blend = "psd_blend"
        
        
        self._name_td_scaler = "td_scaler"
        self._name_td_model = "td_model"
        
        
        self._name_ffc_scaler = "ffc_scaler"
        self._name_ffc_model = "ffc_model"
        
        
        self._name_ar_scaler = "ar_scaler"
        self._name_psd_scaler = "psd_scaler"
        self._name_pca_ar = "pca_ar"
        self._name_pca_psd = "pca_psd"
        
        
        self._name_porosity_scaler = "porosity_scaler"
        self._name_porosity_model = "porosity_model"
        
        self._name_porosity_scaler_wcsd = "porosity_scaler_wcsd"
        self._name_porosity_model_wcsd = "porosity_model_wcsd"
        
        self._name_por_model_data = "por_model_data"
        self._name_por_model_data_wcsd = "por_model_data_wcsd"


        self._name_tensile_scaler = "tensile_scaler"
        self._name_tensile_model = "tensile_model"
        
        self._name_tensile_scaler_wcsd = "tensile_scaler_wcsd"
        self._name_tensile_model_wcsd = "tensile_model_wcsd"
        
        self._name_ts_model_data = "ts_model_data"
        self._name_ts_model_data_wcsd = "ts_model_data_wcsd"
        
        
        # self.models_list = {"psd_scaler": ".pkl",
        #                    "ar_scaler": ".pkl",
        #                    "pca_psd": ".pkl",
        #                    "pca_ar": ".pkl",
        #                    "td_scaler": ".pkl",
        #                    "td_model": ".pkl",
        #                    "ffc_scaler": ".pkl",
        #                    "ffc_model": ".pkl",
        #                    "porosity_scaler": ".pkl",
        #                    "porosity_model": ".h5",
        #                    "porosity_scaler_wcsd": ".pkl",
        #                    "porosity_model_wcsd": ".h5",
        #                    "tensile_scaler": ".pkl",
        #                    "tensile_model": ".h5",
        #                    "tensile_scaler_wcsd": ".pkl",
        #                    "tensile_model_wcsd": ".h5",
        #                    }