# -*- coding: utf-8 -*-
"""
Created on Tue Oct  8 11:25:29 2024

@author: moham
"""
import itertools
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt
from matplotlib.pyplot import figure
# from matplotlib.pyplot import cm
import seaborn as sns
from psynlig import pca_explained_variance_bar
from sklearn.decomposition import PCA
from .config import Config
config = Config()

def get_generic_ID(parts_old, df_raw):
    """
    receives a list of old IDs and returns new list of generic IDs
    """

    parts_new = parts_old
        
    for i, component in enumerate(parts_old):
        if i==len(parts_old)-1:
            parts_new[i] = "blend "+str(component)
        elif component=="xxx":
            parts_new[i] = "XXX"
        else:
            try:
                parts_new[i] = df_raw.loc[df_raw.loc[:,"unique identifier"]==component,"generic identifier"].tolist()[0]
            except:
                parts_new[i] = component
    
    return parts_new
    
def get_generic_blendcode(blend_ID, df_raw):
    """
    receives a blend code with old IDs and returns new blend code with generic IDs
    """
    parts_old = blend_ID.split('_')

    parts_new = parts_old

    parts_new = get_generic_ID(parts_old, df_raw)

    blend_ID_generic = '_'.join(parts_new)

    return blend_ID_generic

def api_based_color(blend_ID, df_raw):
    api = get_generic_blendcode(blend_ID, df_raw).split('_')[0]
    
    api_colors = {
        "SP": "#66c2a5",  # Greenish
        "GRF": "#fc8d62",   # Orange
        "XXX": "#8da0cb",   # Blueish
        "AS": "#e78ac3",   # Pink
        "DEX": "#a6d854",   # Light Green
        "IM": "#ffd92f",   # Yellow
        "MH": "#e5c494",   # Brownish
        "GF": "#b3b3b3",    # Grey
        "MPAR1": "#1f78b4",   # Dark Blue"
        "IBU": "#33a02c",   # Dark Green
        "MF": "#fb9a99",   # Light Red
        "LOV1":"#cab2d6",   # Light Purple
    }
    
    color = api_colors.get(api, "#e78ac3")  # Default pinkish color
    
    return str(color)

def visualise_raw_data (df_raw):
    """
    plots the spread of raw data
    """
    df_raw = df_raw.drop([0])
    
    props = {"True Density": "True Density ($g/cm^3$)", "BD": "Bulk Density ($g/cm^3$)", "TD": "Tapped Density ($g/ml$)", 
             "d10 VB": "$D[v, 0.1]$ $(μm)$", "d50 VB": "$D[v, 0.5]$ $(μm)$", "d90 VB": "$D[v, 0.9]$ $(μm)$", "FFC": "$\overline{FFC}$ $(-)$"}

    fig, axes = plt.subplots(len(props), figsize=(9, 6),
                             sharey='all', constrained_layout=1, dpi=600)
    fig_prop_cycle = plt.rcParams['axes.prop_cycle']()

    for ax, propkey, propval in zip(axes, props.keys(), props.values()):
        temp_data = df_raw[propkey].copy().astype("float")#.dropna()
        ax.scatter(temp_data, [0 for _ in temp_data], s=200, **next(fig_prop_cycle), alpha=0.4)
        for x in 'left top right'.split():ax.spines[x].set_visible(False)
        xmin, xmax = temp_data.min()*0.25, temp_data.max()*1.05
        if propkey=="True Density":
            ax.set_xlim((xmin*3, xmax*1))
        elif propkey=="FFC":
            ax.set_xlim((xmin, xmax))   
        else:
            ax.set_xlim((xmin, xmax))
        # ax.set_xticks(range(xmin,xmax)) 
        # ax.set_xticks(range(25), minor=True)
        ax.set_yticks([]) ; ax.set_yticklabels([])
        ax.set_ylabel(propval, rotation=0, ha='right', va='center', fontsize=12, fontweight='normal')
    plt.show()
    
def summarise_blend_data(df_blend, df_raw):

    df_blend_copy = df_blend.copy()
    df_blend_copy = df_blend_copy.dropna(subset='Blend code')
    
    #generalise blend codes and components to generic identifiers
    for indx in df_blend_copy.index:
        blend_ID = df_blend_copy.loc[indx, "Blend code"]
        df_blend_copy.loc[indx, "Blend code"] = get_generic_blendcode(blend_ID, df_raw)
        old_comps = df_blend_copy.loc[indx, config.blend_comp_list]
        df_blend_copy.loc[indx, config.blend_comp_list] = get_generic_ID(old_comps, df_raw)    
    
    min_max_concentrations = df_blend_copy.groupby('API')['API conc'].agg(['min', 'max']).reset_index()
    print(min_max_concentrations)
    min_max_truedensity = df_blend_copy.groupby('API')['True Density'].agg(['min', 'max']).reset_index()
    print(min_max_truedensity)
    min_max_bd = df_blend_copy.groupby('API')['BD'].agg(['min', 'max']).reset_index()
    print(min_max_bd)
    min_max_psd = df_blend_copy.groupby('API')['d50 VB'].agg(['min', 'max']).reset_index()
    print(min_max_psd)
    min_max_ard = df_blend_copy.groupby('API')['d50 AR'].agg(['min', 'max']).reset_index()
    print(min_max_ard)
    # return df_blend_copy
    
def summarise_tablet_data(df_tablet, df_raw):

    df_tablet_copy = df_tablet.copy()
    df_tablet_copy = df_tablet_copy.dropna(subset='Blend code')
    
    #generalise blend codes and components to generic identifiers
    for indx in df_tablet_copy.index:
        blend_ID = df_tablet_copy.loc[indx, "Blend code"]
        df_tablet_copy.loc[indx, "Blend code"] = get_generic_blendcode(blend_ID, df_raw)
        old_comps = df_tablet_copy.loc[indx, config.blend_comp_list]
        df_tablet_copy.loc[indx, config.blend_comp_list] = get_generic_ID(old_comps, df_raw)    
    
    min_max_concentrations = df_tablet_copy.groupby('API')['API conc'].agg(['min', 'max']).reset_index()
    print(min_max_concentrations)
    min_max_porosity = df_tablet_copy.groupby('API')['Tablet porosity'].agg(['min', 'max']).reset_index()
    print(min_max_porosity)
    min_max_ts = df_tablet_copy.groupby('API')['Tensile strength (Mpa)'].agg(['min', 'max']).reset_index()
    print(min_max_ts)
    min_max_cp = df_tablet_copy.groupby('API')['Compression Pressure (Mpa)'].agg(['min', 'max']).reset_index()
    print(min_max_cp)
    # return df_tablet_copy
    
    # Plotting the distributions all together
    fig, ax = plt.subplots(figsize=(10, 6), dpi=600)
    
    # Tablet porosity (rightmost y-axis)
    sns.kdeplot(df_tablet_copy['Tablet porosity'], ax=ax, color='green', label='Porosity', shade=True, alpha=0.5, clip=(0, 1))
    ax.set_xlabel('Porosity', color='green')
    ax.set_ylabel('Porosity Density', color='green')
    ax.yaxis.set_label_position('left')
    ax.yaxis.tick_right()
    
    # Create a twin axis for Tensile strength (leftmost y-axis)
    ax2 = ax.twinx()
    ax2.spines['left'].set_position(('outward', 60))
    ax2.yaxis.set_label_position('left')
    ax2.yaxis.tick_left()
    ax2_twiny = ax2.twiny()
    sns.kdeplot(df_tablet_copy['Tensile strength (Mpa)'], ax=ax2_twiny, color='orange', label='Tensile Strength (MPa)', shade=True, alpha=0.5, clip=(0, None))
    ax2_twiny.xaxis.set_ticks_position('bottom')
    ax2_twiny.spines['bottom'].set_position(('outward', 40))
    ax2.set_ylabel('Tensile Strength Density', color='orange')
    ax2_twiny.set_xlabel('Tensile Strength (MPa)', color='orange')
    ax2_twiny.xaxis.set_label_position('bottom')

    # Create another twin axis for Compression Pressure (middle y-axis)
    ax3 = ax.twinx()
    ax3.spines['left'].set_position(('outward', 120))
    ax3.yaxis.set_label_position('left')
    ax3.yaxis.tick_left()
    ax3_twiny = ax3.twiny()
    sns.kdeplot(df_tablet_copy['Compression Pressure (Mpa)'], ax=ax3_twiny, color='lightblue', label='Compression Pressure (MPa)', shade=True, alpha=0.5, clip=(0, None))
    ax3_twiny.xaxis.set_ticks_position('bottom')
    ax3_twiny.spines['bottom'].set_position(('outward', 80))
    ax3.set_ylabel('Compression Pressure Density', color='blue')
    ax3_twiny.set_xlabel('Compression Pressure (MPa)', color='blue')
    ax3_twiny.xaxis.set_label_position('bottom')

    # Set x-axis limits
    ax.set_xlim(left=0)
    ax2_twiny.set_xlim(left=0)
    ax3_twiny.set_xlim(left=0)
    
    # Adding legends manually to avoid overlapping labels
    handles, labels = ax.get_legend_handles_labels()
    handles2, labels2 = ax2_twiny.get_legend_handles_labels()
    handles3, labels3 = ax3_twiny.get_legend_handles_labels()
    fig.legend(handles + handles2 + handles3, labels + labels2 + labels3, loc='upper right', bbox_to_anchor=(0.95, 0.95))
    
    plt.tight_layout()
    plt.show()
    
    # plotting distributions separately
    # Tablet porosity (rightmost y-axis)
    fig, ax = plt.subplots(figsize=(6, 4), dpi=600)
    sns.kdeplot(df_tablet_copy['Tablet porosity'], ax=ax, color='lightgreen', label='Porosity', shade=True, alpha=0.5, clip=(0, 1))
    ax.set_xlabel('Porosity (-)', color='black')
    ax.set_ylabel('Porosity Density', color='black')
    ax.set_xlim(left=0)
    plt.tight_layout()
    plt.show()
    
    # Tensile strength
    fig, ax = plt.subplots(figsize=(6, 4), dpi=600)
    sns.kdeplot(df_tablet_copy['Tensile strength (Mpa)'], ax=ax, color='pink', label='Porosity', shade=True, alpha=0.5, clip=(0, None))
    ax.set_xlabel('Tensile Strength (MPa)', color='black')
    ax.set_ylabel('Tensile Strength Density', color='black')
    ax.set_xlim(left=0)
    plt.tight_layout()
    plt.show()
    
    # Compaction pressure
    fig, ax = plt.subplots(figsize=(6, 4), dpi=600)
    sns.kdeplot(df_tablet_copy['Compression Pressure (Mpa)'], ax=ax, color='lightblue', label='Porosity', shade=True, alpha=0.5, clip=(0, None))
    ax.set_xlabel('Compression Pressure (MPa)', color='black')
    ax.set_ylabel('Compression Pressure Density', color='black')
    ax.set_xlim(left=0)
    plt.tight_layout()
    plt.show()
    
    
def show_pc_scores_2d(tablet_index_PSD, PSD_tablet_principal_components, PSD_tablets_transposed, df_raw):
    marker = itertools.cycle((',', 'h', 'v', 'o', '*', 'X', 'D', "P")) 
    fig, ax = plt.subplots(figsize=(10, 10),dpi=300)
    for i in range(len(PSD_tablet_principal_components)):
        ax.scatter(PSD_tablet_principal_components[i, 0], PSD_tablet_principal_components[i, 1], 
                   label=get_generic_blendcode(tablet_index_PSD[i], df_raw), 
                   marker=next(marker), 
                   c=api_based_color(tablet_index_PSD[i], df_raw), 
                   s=80)
    ax.set_xlabel("PC1 Score", fontsize=28)
    ax.set_ylabel("PC2 Score", fontsize=28)
    ax.legend(loc='center left', bbox_to_anchor=(1, 0.5), ncol=2, fontsize='small')
    # ax.set_xlim(20, 40)
    # ax.set_ylim(40, 50)
    ax.grid(False)
    plt.show()
    
    marker = itertools.cycle((',', 'h', 'v', 'o', '*', 'X', 'D', "P")) 
    fig, ax = plt.subplots(figsize=(10, 10),dpi=300)
    for i in range(len(PSD_tablet_principal_components)):
        ax.scatter(PSD_tablet_principal_components[i, 1], PSD_tablet_principal_components[i, 2], 
                   label=get_generic_blendcode(tablet_index_PSD[i], df_raw), 
                   marker=next(marker), 
                   c=api_based_color(tablet_index_PSD[i], df_raw), 
                   s=80)
    ax.set_xlabel("PC2 Score", fontsize=28)
    ax.set_ylabel("PC3 Score", fontsize=28)
    ax.legend(loc='center left', bbox_to_anchor=(1, 0.5), ncol=2, fontsize='small')
    # ax.set_xlim(20, 40)
    # ax.set_ylim(40, 50)
    ax.grid(False)
    plt.show()

    pcamodel_psd_tablet = PCA(n_components=10)
    PSD_tablet_principal_components = pcamodel_psd_tablet.fit_transform(PSD_tablets_transposed)
    figure(dpi=600)
    pca_explained_variance_bar(pcamodel_psd_tablet, color="darkblue" , alpha=0.8)
    plt.xlabel("Principal component", fontsize=12)
    plt.ylabel("Explained variance (fraction)", fontsize=12)
    plt.show()
