import logging
import pandas as pd
from typing import Tuple
from typing_extensions import Annotated
from src.data_cleaning import (
    DataCleaning,)
from src.data_visualisation import (
    visualise_raw_data, 
    summarise_blend_data,
    summarise_tablet_data
    )

def clean_data(df_raw: pd.DataFrame, 
               df_blend: pd.DataFrame, 
               df_tablet: pd.DataFrame) -> Tuple[
                   Annotated[pd.DataFrame, "df_raw"],
                   Annotated[pd.DataFrame, "df_blend"],
                   Annotated[pd.DataFrame, "df_tablet"],
                   ]:
    """
    Data cleaning class which preprocesses the data and divides it into train and test data.

    Args:
        data: pd.DataFrame
    """
    try:
        data_cleaning = DataCleaning()
        df_tablet = data_cleaning.handle_data(df_tablet)
        df_synthetic = data_cleaning.synthetise_data(df_tablet)
        
        # visualise_raw_data(df_raw)
        # summarise_blend_data(df_blend, df_raw)
        # visualise_raw_data(df_blend)
        summarise_tablet_data(df_tablet, df_raw)
        
        return df_raw, df_blend, df_tablet, df_synthetic
    except Exception as e:
        logging.error(e)
        raise e