import logging
from typing import Tuple
from typing_extensions import Annotated

import pandas as pd
from src.data_ingestion import IngestData
from src.config import Config

config = Config()

def ingest_data() -> Tuple[ 
        Annotated[pd.DataFrame, "df_raw"],
        Annotated[pd.DataFrame, "df_blend"],
        Annotated[pd.DataFrame, "df_tablet"],
        Annotated[pd.DataFrame, "psd_raw"],
        Annotated[pd.DataFrame, "ar_raw"],
        Annotated[pd.DataFrame, "psd_blend"],
        Annotated[pd.DataFrame, "ar_blend"],
        ]:
    try:
        ingest_data = IngestData()
        df_raw = ingest_data.get_masterfile(sheet_name = config.raw_data_sheet_name, 
                                            path = config.raw_data_path, 
                                            skiprows = config.raw_data_skip_rows)
        logging.info('raw masterfile loaded.')
        df_blend = ingest_data.get_masterfile(sheet_name = config.blend_data_sheet_name, 
                                            path = config.blend_data_path, 
                                            skiprows = config.blend_data_skip_rows)
        logging.info('blend masterfile loaded.')
        df_tablet = ingest_data.get_masterfile(sheet_name = config.tablet_data_sheet_name, 
                                            path = config.tablet_data_path, 
                                            skiprows = config.tablet_data_skip_rows)
        logging.info('tablet masterfile loaded.')
        psd_raw = ingest_data.get_raw_particle_data (descriptor="CE Diameter (µm)", 
                                                     folder_path=config.raw_PSD_num_path, 
                                                     df=df_raw)
        logging.info('psd raw data loaded.')
        ar_raw = ingest_data.get_raw_particle_data (descriptor="Aspect Ratio", 
                                                    folder_path=config.raw_AR_path, 
                                                    df=df_raw)
        logging.info('aspect ratio raw data loaded.')
        psd_blend = ingest_data.get_blend_particle_data (descriptor="CE Diameter (µm)", 
                                                         folder_path=config.blend_PSD_num_path, 
                                                         df=df_blend)
        logging.info('psd blend data loaded.')
        ar_blend = ingest_data.get_blend_particle_data (descriptor="Aspect Ratio", 
                                                        folder_path=config.blend_AR_path, 
                                                        df=df_blend)
        logging.info('aspect ratio blend data loaded.')
        return df_raw, df_blend, df_tablet, psd_raw, ar_raw, psd_blend, ar_blend
    except Exception as e:
        logging.error(e)
        raise e

def load_models(folder_path: str, model_name: str, model_suffix: str):
    try:
        ingest_data = IngestData()
        model = ingest_data.load_model(folder_path, model_name, model_suffix)
        return model
    except Exception as e:
        logging.error(e)
        raise e